/*
 * encoder.cpp
 */

#include <stm32f4xx.h>
#include "encoder.h"
#include "gpio.h"

TIM_TypeDef * tim_enc = TIM3;

void init_encoder(void)
{

    /* Enable GPIOA*/
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;

	
	/************************************  ENCODER    ****************************************/
	/*Configure GPIO for encoder */
	GPIO_INPUT(GPIOA, 6);
	GPIO_ALTERNATE(GPIOA,6);
	GPIO_NO_PUPD(GPIOA, 6);
	GPIO_AFL(GPIOA, 6, AF_TIM3_5);
	
	GPIO_INPUT(GPIOA, 7);
	GPIO_ALTERNATE(GPIOA, 7);
	GPIO_NO_PUPD(GPIOA, 7);
	GPIO_AFL(GPIOA, 7, AF_TIM3_5);
	
	/*Select and configure the timer input:*/
	RCC->APB1ENR |= RCC_APB1ENR_TIM3EN;
	tim_enc->PSC = 0;
	tim_enc->ARR = 0xFFFF; // 16bit
	
	/*Input selection*/
	tim_enc->CCMR1 = 0x0101;
    // input 1 & 2 in Capture MODE
    // 0000 00 01 0000 00 01 = 0x0101;
    //    |  |  |    |  |  |
    //    |  |  |    |  |  +---- IC1 on T1
    //    |  |  |    |  +------- capture every edge
    //    |  |  |    +---------- no filter
    //    |  |  +--------------- IC2 on T2
    //    |  +------------------ capture every edge
    //    +--------------------- no filter
	
	
	/*Input Polarity*/
	tim_enc->CCER &= ~(TIM_CCER_CC1P);
	tim_enc->CCER &= ~(TIM_CCER_CC1NP);
	tim_enc->CCER &= ~(TIM_CCER_CC2P);
	tim_enc->CCER &= ~(TIM_CCER_CC2NP);
	
	
	/*Encoder mode: 3 (SMS = 011)*/
	tim_enc->SMCR &= ~TIM_SMCR_SMS;
	tim_enc->SMCR |= (TIM_SMCR_SMS_0 | TIM_SMCR_SMS_1);
	
    start_encoder();

}

void reset_encoder(void)
{
	tim_enc->CNT = 0;
}

void start_encoder(void)
{
	tim_enc->CNT = 0;
	tim_enc->CR1 |= TIM_CR1_CEN;
}


void stop_encoder(void)
{
    tim_enc->CNT = 0;
    tim_enc->CR1 &= ~TIM_CR1_CEN;
}

short read_encoder(void)
{
	return tim_enc->CNT;
}
